from __future__ import annotations

from typing import TYPE_CHECKING, Dict

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, SyncHttpClient


class Usage:
    """Sync usage resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def get(self, params: Dict | None = None) -> APIResponse:
        return self._client.get("/v1/me/usage", query=params)


class AsyncUsage:
    """Async usage resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def get(self, params: Dict | None = None) -> APIResponse:
        return await self._client.get("/v1/me/usage", query=params)
