from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, SyncHttpClient


class Conflicts:
    """Sync conflicts resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def list(self) -> APIResponse:
        return self._client.get("/v1/conflicts")


class AsyncConflicts:
    """Async conflicts resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def list(self) -> APIResponse:
        return await self._client.get("/v1/conflicts")
